#include "Config.h"

// Software date and version
static String Config::DATE                               = "2024-09-22";
static String Config::VERSION                            = "V2.1.0";
// Lines to scroll in lists when up or down is pressed.
static unsigned char Config::SCROLL_WIDTH                = 6;
// Number of Lines in list views.
static unsigned char Config::LIST_SIZE                   = 6;
// FILENAME der Config Tabelle auf der SD Karte
static String Config::FILENAME_CONFIG                    = "conf.csv";
// Initial Freq (on no CI-V data)
static unsigned long Config::INITIAL_FREQUENCY           = 14200000;
// Display Brightness 0...100
static unsigned char     Config::BRIGHTNESS              = 100;
// Interval in milliseconds of requesting
// frequency from transceiver
// 0 - No automatic requests
static unsigned long Config::REQUEST_INTERVAL            = 1000;
// Period in milliseconds to restart request of transceiver data.
static unsigned long Config::RESTART_DELAY               = 5000;
// Controller Addresse (HEX)
static unsigned char     Config::CONTROLLER_ADDR         = 0xAA;
// Transceiver CI-V Adresse (HEX)
static unsigned char     Config::CIV_ADDR                = 0x7A;
// Inverts output BIT-Pattern on Koppler
static bool Config::INVERT_KOPPLER_OUTPUT_L              = false;
static bool Config::INVERT_KOPPLER_OUTPUT_C              = false;
static bool Config::INVERT_KOPPLER_OUTPUT_IO             = false;
// L C
static int Config::STEP_LC_INITIAL_VALUE                 = 1;
// FREQUENCY
static int Config::STEP_FREQUENCY_INITIAL_VALUE          = 10;
static byte Config::STEP_BIT_WIDTH_L                     = 11;
static byte Config::STEP_BIT_WIDTH_C                     = 11;
static float Config::STEP_C_MIN_VALUE                    = 3.125; 
static float Config::STEP_L_MIN_VALUE                    = 40; 
static byte Config::PIN_ONEWIRE                          = 23;
// FILENAME table on SDcard
static String Config::FILENAME_FREQ                      = "freqtab.csv";
// CS Arduino Pin 53
static byte Config::CHIP_SELECT_PIN                      = 53;
// Adress IC1 DS2408S 
// Address Testboard: 0x29,0x22,0x8E,0x1B,0x00,0x00,0x00,0x3E
static byte Config::IC1[8] = {0x29,0x22,0x8E,0x1B,0x00,0x00,0x00,0x3E};//{0x29,0x7A,0x8F,0x1B,0x00,0x00,0x00,0x7C};
// Adress IC2 DS2408S
static byte Config::IC2[8] = {0x29,0x22,0x8E,0x1B,0x00,0x00,0x00,0x3E};//{0x29,0x8A,0x8E,0x1B,0x00,0x00,0x00,0xC3};
// Adress IC3 DS2408S
static byte Config::IC3[8] = {0x29,0x22,0x8E,0x1B,0x00,0x00,0x00,0x3E};//{0x29,0x2A,0x8E,0x1B,0x00,0x00,0x00,0x9F};

static unsigned long Config::SERIAL_SPEED_SERIAL1 = 9600;
static unsigned long Config::SERIAL_SPEED_SERIAL2 = 9600;
static unsigned long Config::SERIAL_SPEED_SERIAL3 = 9600;

static String Config::NAME_PAGE_SPLASH      = "page 0";
static String Config::NAME_PAGE_WAIT        = "page 1";
static String Config::NAME_PAGE_AUTO_LEFT   = "page 2";
static String Config::NAME_PAGE_AUTO_RIGHT  = "page 3";
static String Config::NAME_PAGE_FREQ_LEFT   = "page 4";
static String Config::NAME_PAGE_FREQ_RIGHT  = "page 5";
static String Config::NAME_PAGE_MAN_C_LEFT  = "page 6";
static String Config::NAME_PAGE_MAN_C_RIGHT = "page 7";
static String Config::NAME_PAGE_SETUP       = "page 8";
static String Config::NAME_PAGE_KEYBOARD    = "page 9";

static unsigned char Config::KEY_CAPACITANCE_UP      = 49;
static unsigned char Config::KEY_CAPACITANCE_DOWN    = 50;
static unsigned char Config::KEY_INDUCTANCE_UP       = 51;
static unsigned char Config::KEY_INDUCTANCE_DOWN     = 52;
static unsigned char Config::KEY_SWITCH_CAP_TRX      = 55;
static unsigned char Config::KEY_STEP                = 54;
static unsigned char Config::KEY_SWITCH_CAP_ANT      = 53;
static unsigned char Config::KEY_FREQUENCY_UP        = 56;
static unsigned char Config::KEY_FREQUENCY_DOWN      = 57;
static unsigned char Config::KEY_A                   = 65;
static unsigned char Config::KEY_B                   = 66;
static unsigned char Config::KEY_C                   = 67;
static unsigned char Config::KEY_D                   = 68;
static unsigned char Config::KEY_E                   = 69;
static unsigned char Config::KEY_F                   = 70;
static unsigned char Config::KEY_0                   = 71;
static unsigned char Config::KEY_1                   = 72;
static unsigned char Config::KEY_2                   = 73;
static unsigned char Config::KEY_3                   = 74;
static unsigned char Config::KEY_4                   = 75;
static unsigned char Config::KEY_5                   = 76;
static unsigned char Config::KEY_6                   = 77;
static unsigned char Config::KEY_7                   = 78;
static unsigned char Config::KEY_8                   = 79;
static unsigned char Config::KEY_9                   = 80;
static unsigned char Config::KEY_SPACE               = 88;
static unsigned char Config::KEY_BACK                = 89;
static unsigned char Config::KEY_SEXIT               = 90;
static unsigned char Config::KEY_FREQUENCY_STEP      = 97;
static unsigned char Config::KEY_AUTO                = 98;
static unsigned char Config::KEY_MAN1                = 99;
static unsigned char Config::KEY_MAN2                = 100;
static unsigned char Config::KEY_STORE               = 101;
static unsigned char Config::KEY_SETUP               = 102;
static unsigned char Config::KEY_SSTORE              = 103;
static unsigned char Config::KEY_EXIT                = 104;
static unsigned char Config::KEY_SNEXT               = 105;
static unsigned char Config::KEY_SPREV               = 106;
static unsigned char Config::KEY_RESET               = 107;
static unsigned char Config::KEY_EDIT0               = 108;
static unsigned char Config::KEY_EDIT1               = 109;
static unsigned char Config::KEY_EDIT2               = 110;
static unsigned char Config::KEY_EDIT3               = 111;
static unsigned char Config::KEY_EDIT4               = 112;
static unsigned char Config::KEY_EDIT5               = 113;


// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//
// DON'T CHANGE FILE BELOW THIS LINE
// 
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

Config::Config(){}

static String Config::getPart(String cmd, unsigned char part){
  String delimiter = " ";
  int posStart = 0;
  int posEnd = cmd.length();

  unsigned char partCount = 0;
  for( int i = 0; i <= cmd.length(); i++){
    if( cmd.substring(i,i+1) != delimiter){
      posStart = i;
      if(partCount == part){
        break;
      }
    }
    if( cmd.substring(i,i+1) == delimiter)
      partCount++;
  }

  for( int i = posStart+1; i < cmd.length(); i++){
    if( cmd.substring(i,i+1) == delimiter){
      posEnd = i;
      break;
    }
  }
  if( posStart >= posEnd)
    return "";

  return cmd.substring(posStart, posEnd);

}

// The file name if the csv file on the csv file.
static String Config::fileName;
// The line break character in the csv file.
static char Config::csvLineBreak;

void Config::init( String _fileName ){
  fileName = _fileName;
  csvLineBreak = char(13);
}

// Returns the number of entries in the Config list.
static int Config::getConfigSize(){
  return 8;
}

// Returns a list with labels of the config values.
static String *Config::getConfigLabels(){
  static String result[9];

  result[0] = "initial_frequency";
  result[1] = "request_interval";
  result[2] = "brightness";
  result[3] = "restart_delay";
  result[4] = "controller_addr";
  result[5] = "civ_addr";
  result[6] = "scroll_width";
  result[7] = "list_size";

  return result;
}

// Returns a list with the actual config values as string
static String *Config::getConfigValues(){
  static String result[8];
  result[0] = String(Config::INITIAL_FREQUENCY);
  result[1] = String(Config::REQUEST_INTERVAL);
  result[2] = String(Config::BRIGHTNESS);
  result[3] = String(Config::RESTART_DELAY);
  result[4] = String(Config::CONTROLLER_ADDR, HEX);
  result[4].toUpperCase();
  result[5] = String(Config::CIV_ADDR, HEX);
  result[5].toUpperCase();
  result[6] = String(Config::SCROLL_WIDTH);
  result[7] = String(Config::LIST_SIZE);
  return result;
}

static unsigned long Config::toRange(String value, unsigned long min=0, unsigned long max=1000000000){
  
  String canditate = "";
  // remove all Letters
  for(int i = 0; i < value.length(); i++)
  {
    String test = value.substring(i,i+1);
    if( 
        test == "0" ||
        test == "1" ||
        test == "2" || 
        test == "3" || 
        test == "4" || 
        test == "5" ||
        test == "6" || 
        test == "7" ||
        test == "8" ||
        test == "9"
    ){
      canditate += test;
    }
  }
  
  unsigned long result =  (unsigned long)canditate.toInt();

  return max(min,min(result,max));

}

static unsigned char Config::toAddr1(String value){
  
  // remove all leading 0
  while(true)
  {
    if( value.substring(0,1) == "0")
    {
      value = value.substring(1);
    }
    else
    {
      break;
    }
  }

  // add one leading 0 if necessary
  if( value.length() < 2)
    value = "0"+value;

  unsigned char tens = hex2Byte(value.substring(0,1) );
  unsigned char ones = hex2Byte(value.substring(1,2) );
  return (16 * tens) + ones;  

}

static bool Config::toBool(String value){
  return value == "1";
}

// Sets a config value.
static bool Config::setConfigValue(String label, String value){
  
  if(label == "initial_frequency"){
    Config::INITIAL_FREQUENCY = Config::toRange(value ,0 ,999999999);
  }else if(label == "request_interval"){
    Config::REQUEST_INTERVAL = Config::toRange(value ,0 ,10000);
  }else if(label == "brightness"){
    Config::BRIGHTNESS = Config::toRange(value ,5 ,100);
  }else if(label == "restart_delay"){
    Config::RESTART_DELAY = Config::toRange(value ,0 ,10000);
  }else if(label == "controller_addr"){
    Config::CONTROLLER_ADDR = Config::toAddr1(value);
  }else if(label == "civ_addr"){
    Config::CIV_ADDR = Config::toAddr1(value);
  }else if(label == "inv_coupler_out_c"){
    Config::INVERT_KOPPLER_OUTPUT_C = Config::toBool(value);
  }else if(label == "inv_coupler_out_l"){
    Config::INVERT_KOPPLER_OUTPUT_L = Config::toBool(value);
  }else if(label == "inv_coupler_out_io"){
    Config::INVERT_KOPPLER_OUTPUT_IO = Config::toBool(value);
  }else if(label == "scroll_width"){
    Config::SCROLL_WIDTH = Config::toRange(value,0,64);
  }else if(label == "list_size"){
    Config::LIST_SIZE = Config::toRange(value,0,64);
  }else if(label == "step_bit_width_l"){
    Config::STEP_BIT_WIDTH_L = Config::toRange(value,0,64);
  }else if(label == "step_bit_width_c"){
    Config::STEP_BIT_WIDTH_C = Config::toRange(value,0,64);
  }else if(label == "step_c_min_value"){
    Config::STEP_C_MIN_VALUE = value.toFloat();
  }else if(label == "step_l_min_value"){
    Config::STEP_L_MIN_VALUE = value.toFloat();
  }else if(label == "pin_onewire"){
    Config::PIN_ONEWIRE = Config::toRange(value,0,64);
  }else if(label == "step_lc_initial_value"){
    Config::STEP_LC_INITIAL_VALUE = Config::toRange(value,0,100);
  }else if(label == "step_frequency_initial_value"){
    Config::STEP_FREQUENCY_INITIAL_VALUE = Config::toRange(value,0,1000);
  }else if(label == "chip_select_pin"){
    Config::CHIP_SELECT_PIN = Config::toRange(value,0,100);
  }else if(label == "serial_speed_serial1"){
    Config::SERIAL_SPEED_SERIAL1 = Config::toRange(value,0,999999999);
  }else if(label == "serial_speed_serial2"){
    Config::SERIAL_SPEED_SERIAL2 = Config::toRange(value,0,999999999);
  }else if(label == "serial_speed_serial3"){
    Config::SERIAL_SPEED_SERIAL2 = Config::toRange(value,0,999999999);
  }
  
  return true;

}

// 0x00 - 0xff
static int Config::hex2Byte(String hex){
  hex.toLowerCase();
  if(hex.length() == 1){
    if(hex == "0") return 0;
    if(hex == "1") return 1;
    if(hex == "2") return 2;
    if(hex == "3") return 3;
    if(hex == "4") return 4;
    if(hex == "5") return 5;
    if(hex == "6") return 6;
    if(hex == "7") return 7;
    if(hex == "8") return 8; 
    if(hex == "9") return 9;
    if(hex == "a") return 10;
    if(hex == "b") return 11;
    if(hex == "c") return 12;
    if(hex == "d") return 13;
    if(hex == "e") return 14;
    if(hex == "f") return 15;
  }
  return (hex2Byte(hex.substring(0,1)) << 4) | hex2Byte(hex.substring(1,2));
}

void Config::load(){
  
  File file = SD.open(fileName,FILE_READ);
  if (!file) {
    Serial.println("Config::load(): File couldn't be opened!");
    return false;
  }

  while (file.available()) {

    String line = file.readStringUntil(csvLineBreak);
    line.trim();
    int comma1 = line.indexOf("=");
    String key = line.substring(0, comma1);
    String value = line.substring(comma1+1, line.length() );
    
    if( key == "LIST_SIZE")
      setConfigValue("list_size",value);
    else if( key == "SCROLL_WIDTH")
      setConfigValue("scroll_width",value);
    else if( key == "INITIAL_FREQUENCY")
      setConfigValue("initial_frequency",value);
    else if( key == "BRIGHTNESS")
      setConfigValue("brightness",value);
    else if( key == "REQUEST_INTERVAL")
      setConfigValue("request_interval",value);
    else if( key == "RESTART_DELAY")
      setConfigValue("restart_delay",value);
    else if( key == "CONTROLLER_ADDR")
      setConfigValue("controller_addr",value);
    else if( key == "CIV_ADDR")
      setConfigValue("civ_addr",value);
    else if( key == "INVERT_KOPPLER_OUTPUT_L")
      setConfigValue("inv_coupler_out_l",value);
    else if( key == "INVERT_KOPPLER_OUTPUT_C")
      setConfigValue("inv_coupler_out_c",value);
    else if( key == "INVERT_KOPPLER_OUTPUT_IO")
      setConfigValue("inv_coupler_out_io",value);
    else if( key == "STEP_LC_INITIAL_VALUE")
      setConfigValue("step_lc_initial_value",value);
    else if( key == "STEP_BIT_WIDTH_l")
      setConfigValue("step_bit_width_l",value);
    else if( key == "STEP_BIT_WIDTH_C")
      setConfigValue("step_bit_width_c",value);
    else if( key == "STEP_C_MIN_VALUE")
      setConfigValue("step_c_min_value",value);
    else if( key == "STEP_L_MIN_VALUE")
      setConfigValue("step_l_min_value",value);
    else if( key == "STEP_FREQUENCY_INITIAL_VALUE")
      setConfigValue("step_frequency_initial_value",value);
    else if( key == "PIN_ONEWIRE")
      setConfigValue("pin_onewire",value);
    else if( key == "FILENAME_FREQ")
      setConfigValue("",value);
    else if( key == "CHIP_SELECT_PIN")
      setConfigValue("chip_select_pin",value);
    else if( key == "IC1"){
      IC1[0] = hex2Byte(getPart(value,0));
      IC1[1] = hex2Byte(getPart(value,1));
      IC1[2] = hex2Byte(getPart(value,2));
      IC1[3] = hex2Byte(getPart(value,3));
      IC1[4] = hex2Byte(getPart(value,4));
      IC1[5] = hex2Byte(getPart(value,5));
      IC1[6] = hex2Byte(getPart(value,6));
      IC1[7] = hex2Byte(getPart(value,7));
    }else if( key == "IC2"){
      IC2[0] = hex2Byte(getPart(value,0));
      IC2[1] = hex2Byte(getPart(value,1));
      IC2[2] = hex2Byte(getPart(value,2));
      IC2[3] = hex2Byte(getPart(value,3));
      IC2[4] = hex2Byte(getPart(value,4));
      IC2[5] = hex2Byte(getPart(value,5));
      IC2[6] = hex2Byte(getPart(value,6));
      IC2[7] = hex2Byte(getPart(value,7));
    }else if( key == "IC3"){
      IC3[0] = hex2Byte(getPart(value,0));
      IC3[1] = hex2Byte(getPart(value,1));
      IC3[2] = hex2Byte(getPart(value,2));
      IC3[3] = hex2Byte(getPart(value,3));
      IC3[4] = hex2Byte(getPart(value,4));
      IC3[5] = hex2Byte(getPart(value,5));
      IC3[6] = hex2Byte(getPart(value,6));
      IC3[7] = hex2Byte(getPart(value,7));
    }else if( key == "SERIAL_SPEED_SERIAL1")
      setConfigValue("serial_speed_serial1",value);
    else if( key == "SERIAL_SPEED_SERIAL2")
      setConfigValue("serial_speed_serial2",value);
    else if( key == "SERIAL_SPEED_SERIAL3")
      setConfigValue("serial_speed_serial3",value);
    /*
    else if( key == "NAME_PAGE_SPLASH")
      NAME_PAGE_SPLASH = value;
    else if( key == "NAME_PAGE_WAIT")
      NAME_PAGE_WAIT = value;
    else if( key == "NAME_PAGE_AUTO_LEFT")
      NAME_PAGE_AUTO_LEFT = value;
    else if( key == "NAME_PAGE_AUTO_RIGHT")
      NAME_PAGE_AUTO_RIGHT = value;
    else if( key == "NAME_PAGE_FREQ_LEFT")
      NAME_PAGE_FREQ_LEFT = value;
    else if( key == "NAME_PAGE_FREQ_RIGHT")
      NAME_PAGE_FREQ_RIGHT = value;
    else if( key == "NAME_PAGE_MAN_C_LEFT")
      NAME_PAGE_MAN_C_LEFT = value;
    else if( key == "NAME_PAGE_MAN_C_RIGHT")
      NAME_PAGE_MAN_C_RIGHT = value;
    else if( key == "NAME_PAGE_SETUP")
      NAME_PAGE_SETUP = value;
    else if( key == "NAME_PAGE_LOG")
      NAME_PAGE_LOG = value;
    else if( key == "NAME_PAGE_KEYBOARD")
      NAME_PAGE_KEYBOARD = value;

    else if( key == "KEY_CAPACITANCE_UP")
      KEY_CAPACITANCE_UP = value.toInt();
    else if( key == "KEY_CAPACITANCE_DOWN")
      KEY_CAPACITANCE_DOWN = value.toInt();
    else if( key == "KEY_INDUCTANCE_UP")
      KEY_INDUCTANCE_UP = value.toInt();
    else if( key == "KEY_INDUCTANCE_DOWN")
      KEY_INDUCTANCE_DOWN = value.toInt();
    else if( key == "KEY_SWITCH_CAP_TRX")
      KEY_SWITCH_CAP_TRX = value.toInt();
    else if( key == "KEY_STEP")
      KEY_STEP = value.toInt();
    else if( key == "KEY_SWITCH_CAP_ANT")
      KEY_SWITCH_CAP_ANT = value.toInt();
    else if( key == "KEY_FREQUENCY_UP")
      KEY_FREQUENCY_UP = value.toInt();
    else if( key == "KEY_FREQUENCY_DOWN")
      KEY_FREQUENCY_DOWN = value.toInt();
    else if( key == "KEY_A")
      KEY_A = value.toInt();   
    else if( key == "KEY_B")
      KEY_B = value.toInt();     
    else if( key == "KEY_C")
      KEY_C = value.toInt();   
    else if( key == "KEY_D")
      KEY_D = value.toInt(); 
    else if( key == "KEY_E")
      KEY_E = value.toInt();   
    else if( key == "KEY_F")
      KEY_F = value.toInt(); 
    else if( key == "KEY_0")
      KEY_0 = value.toInt();   
    else if( key == "KEY_1")
      KEY_1 = value.toInt();  
    else if( key == "KEY_2")
      KEY_2 = value.toInt();  
    else if( key == "KEY_3")
      KEY_3 = value.toInt();  
    else if( key == "KEY_4")
      KEY_4 = value.toInt();  
    else if( key == "KEY_5")
      KEY_5 = value.toInt();  
    else if( key == "KEY_6")
      KEY_6 = value.toInt();  
    else if( key == "KEY_7")
      KEY_7 = value.toInt();  
    else if( key == "KEY_8")
      KEY_8 = value.toInt();  
    else if( key == "KEY_9")
      KEY_9 = value.toInt();  
    else if( key == "KEY_SPACE")
      KEY_SPACE = value.toInt();
    else if( key == "KEY_BACK")
      KEY_BACK = value.toInt(); 
    else if( key == "KEY_SEXIT")
      KEY_SEXIT = value.toInt();   
    else if( key == "KEY_FREQUENCY_STEP")
      KEY_FREQUENCY_STEP = value.toInt();
    else if( key == "KEY_AUTO")
      KEY_AUTO = value.toInt();
    else if( key == "KEY_MAN1")
      KEY_MAN1 = value.toInt();
    else if( key == "KEY_MAN2")
      KEY_MAN2 = value.toInt();
    else if( key == "KEY_STORE")
      KEY_STORE = value.toInt();
    else if( key == "KEY_SETUP")
      KEY_SETUP = value.toInt();
    else if( key == "KEY_SSTORE")
      KEY_SSTORE = value.toInt();
    else if( key == "KEY_EXIT")
      KEY_EXIT = value.toInt();
    else if( key == "KEY_SNEXT")
      KEY_SNEXT = value.toInt();
    else if( key == "KEY_SPREV")
      KEY_SPREV = value.toInt();
    else if( key == "KEY_LOG")
      KEY_LOG = value.toInt();    
    else if( key == "KEY_EDIT0")
      KEY_EDIT0 = value.toInt();    
    else if( key == "KEY_EDIT1")
      KEY_EDIT1 = value.toInt();    
    else if( key == "KEY_EDIT2")
      KEY_EDIT2 = value.toInt();    
    else if( key == "KEY_EDIT3")
      KEY_EDIT3 = value.toInt();    
    else if( key == "KEY_EDIT4")
      KEY_EDIT4 = value.toInt();    
    else if( key == "KEY_EDIT5")
      KEY_EDIT5 = value.toInt();    
    else if( key == "KEY_LNEXT")
      KEY_LNEXT = value.toInt();
    else if( key == "KEY_LPREV")
      KEY_LPREV = value.toInt();
    /**/
  }

  file.close();
  
}

void Config::save(){
  
  // delete file
  if( SD.exists(fileName) ){
    SD.remove(fileName);
    Serial.println("Config::save(): Configfile deleted!");
  }

  File file = SD.open(fileName,FILE_WRITE);
  if (!file) {
    Serial.println("Config::save(): Configfile couldn't be created!");
    return;
  }

  file.print( "LIST_SIZE="+String(LIST_SIZE)+csvLineBreak );
  file.print( "SCROLL_WIDTH="+String(SCROLL_WIDTH)+csvLineBreak );
  file.print( "INITIAL_FREQUENCY="+String(INITIAL_FREQUENCY)+csvLineBreak );
  file.print( "BRIGHTNESS="+String(BRIGHTNESS)+csvLineBreak );
  file.print( "REQUEST_INTERVAL="+String(REQUEST_INTERVAL)+csvLineBreak );
  file.print( "RESTART_DELAY="+String(RESTART_DELAY)+csvLineBreak );
  file.print( "CONTROLLER_ADDR="+String(CONTROLLER_ADDR,16)+csvLineBreak );
  file.print( "CIV_ADDR="+String(CIV_ADDR,16)+csvLineBreak );
  String v = "0";
  if(INVERT_KOPPLER_OUTPUT_C)
    v = "1";
  file.print( "INVERT_KOPPLER_OUTPUT_C="+v+csvLineBreak );
  
  v = "0";
  if(INVERT_KOPPLER_OUTPUT_L)
    v = "1";
  file.print( "INVERT_KOPPLER_OUTPUT_L="+v+csvLineBreak );

  v = "0";
  if(INVERT_KOPPLER_OUTPUT_IO)
    v = "1";
  file.print( "INVERT_KOPPLER_OUTPUT_IO="+v+csvLineBreak );

  file.print( "STEP_LC_INITIAL_VALUE="+String(STEP_LC_INITIAL_VALUE)+csvLineBreak );
  file.print( "STEP_BIT_WIDTH_L="+String(STEP_BIT_WIDTH_L)+csvLineBreak );
  file.print( "STEP_BIT_WIDTH_C="+String(STEP_BIT_WIDTH_C)+csvLineBreak );
  file.print( "STEP_C_MIN_VALUE="+String(STEP_C_MIN_VALUE,3)+csvLineBreak );
  file.print( "STEP_L_MIN_VALUE="+String(STEP_L_MIN_VALUE,3)+csvLineBreak );
  file.print( "STEP_FREQUENCY_INITIAL_VALUE="+String(STEP_FREQUENCY_INITIAL_VALUE)+csvLineBreak );
  file.print( "PIN_ONEWIRE="+String(PIN_ONEWIRE)+csvLineBreak );
  file.print( "FILENAME_FREQ="+String(FILENAME_FREQ)+csvLineBreak );
  file.print( "CHIP_SELECT_PIN="+String(CHIP_SELECT_PIN)+csvLineBreak );
  
  String data = "";
  for(unsigned char i = 0; i < 8; i++)
    data += String(IC1[i],HEX)+" ";
  file.print( "IC1="+data +csvLineBreak );

  data = "";
  for(unsigned char i = 0; i < 8; i++)
    data += String(IC2[i],HEX)+" ";
  file.print( "IC2="+data +csvLineBreak );

  data = "";
  for(unsigned char i = 0; i < 8; i++)
    data += String(IC3[i],HEX)+" ";
  file.print( "IC3="+data +csvLineBreak );
  /*
  file.print( "SERIAL_SPEED_SERIAL1="+String(SERIAL_SPEED_SERIAL1)+csvLineBreak );
  file.print( "SERIAL_SPEED_SERIAL2="+String(SERIAL_SPEED_SERIAL2)+csvLineBreak );
  file.print( "SERIAL_SPEED_SERIAL3="+String(SERIAL_SPEED_SERIAL3)+csvLineBreak );
  file.print( "NAME_PAGE_SPLASH="+String(NAME_PAGE_SPLASH)+csvLineBreak );
  file.print( "NAME_PAGE_WAIT="+String(NAME_PAGE_WAIT)+csvLineBreak );
  file.print( "NAME_PAGE_AUTO_LEFT="+String(NAME_PAGE_AUTO_LEFT)+csvLineBreak );
  file.print( "NAME_PAGE_AUTO_RIGHT="+String(NAME_PAGE_AUTO_RIGHT)+csvLineBreak );
  file.print( "NAME_PAGE_FREQ_LEFT="+String(NAME_PAGE_FREQ_LEFT)+csvLineBreak );
  file.print( "NAME_PAGE_FREQ_RIGHT="+String(NAME_PAGE_FREQ_RIGHT)+csvLineBreak );
  file.print( "NAME_PAGE_MAN_C_LEFT="+String(NAME_PAGE_MAN_C_LEFT)+csvLineBreak );
  file.print( "NAME_PAGE_MAN_C_RIGHT="+String(NAME_PAGE_MAN_C_RIGHT)+csvLineBreak );
  file.print( "NAME_PAGE_SETUP="+String(NAME_PAGE_SETUP)+csvLineBreak );
  file.print( "NAME_PAGE_LOG="+String(NAME_PAGE_LOG)+csvLineBreak );
  file.print( "NAME_PAGE_KEYBOARD="+String(NAME_PAGE_KEYBOARD)+csvLineBreak );

  file.print( "KEY_CAPACITANCE_UP="+String(KEY_CAPACITANCE_UP)+csvLineBreak );
  file.print( "KEY_CAPACITANCE_DOWN="+String(KEY_CAPACITANCE_DOWN)+csvLineBreak );
  file.print( "KEY_INDUCTANCE_UP="+String(KEY_INDUCTANCE_UP)+csvLineBreak );
  file.print( "KEY_INDUCTANCE_DOWN="+String(KEY_INDUCTANCE_DOWN)+csvLineBreak );
  file.print( "KEY_SWITCH_CAP_TRX="+String(KEY_SWITCH_CAP_TRX)+csvLineBreak );
  file.print( "KEY_STEP="+String(KEY_STEP)+csvLineBreak );
  file.print( "KEY_SWITCH_CAP_ANT="+String(KEY_SWITCH_CAP_ANT)+csvLineBreak );
  file.print( "KEY_FREQUENCY_UP="+String(KEY_FREQUENCY_UP)+csvLineBreak );
  file.print( "KEY_FREQUENCY_DOWN="+String(KEY_FREQUENCY_DOWN)+csvLineBreak );
  
  file.print( "KEY_A="+String(KEY_A)+csvLineBreak );
  file.print( "KEY_B="+String(KEY_B)+csvLineBreak );
  file.print( "KEY_C="+String(KEY_C)+csvLineBreak );
  file.print( "KEY_D="+String(KEY_D)+csvLineBreak );
  file.print( "KEY_E="+String(KEY_E)+csvLineBreak );
  file.print( "KEY_F="+String(KEY_F)+csvLineBreak );
  file.print( "KEY_0="+String(KEY_0)+csvLineBreak );
  file.print( "KEY_1="+String(KEY_1)+csvLineBreak );
  file.print( "KEY_2="+String(KEY_2)+csvLineBreak );
  file.print( "KEY_3="+String(KEY_3)+csvLineBreak );
  file.print( "KEY_4="+String(KEY_4)+csvLineBreak );
  file.print( "KEY_5="+String(KEY_5)+csvLineBreak );
  file.print( "KEY_6="+String(KEY_6)+csvLineBreak );
  file.print( "KEY_7="+String(KEY_7)+csvLineBreak );
  file.print( "KEY_8="+String(KEY_8)+csvLineBreak );
  file.print( "KEY_9="+String(KEY_9)+csvLineBreak );
  file.print( "KEY_SPACE="+String(KEY_SPACE)+csvLineBreak );
  file.print( "KEY_BACK="+String(KEY_BACK)+csvLineBreak );
  file.print( "KEY_SEXIT="+String(KEY_SEXIT)+csvLineBreak );
  file.print( "KEY_FREQUENCY_STEP="+String(KEY_FREQUENCY_STEP)+csvLineBreak );  
  file.print( "KEY_AUTO="+String(KEY_AUTO)+csvLineBreak );
  file.print( "KEY_MAN1="+String(KEY_MAN1)+csvLineBreak );
  file.print( "KEY_MAN2="+String(KEY_MAN2)+csvLineBreak );
  file.print( "KEY_STORE="+String(KEY_STORE)+csvLineBreak );
  file.print( "KEY_SETUP="+String(KEY_SETUP)+csvLineBreak );
  file.print( "KEY_SSTORE="+String(KEY_SSTORE)+csvLineBreak );
  file.print( "KEY_EXIT="+String(KEY_EXIT)+csvLineBreak );
  file.print( "KEY_SNEXT="+String(KEY_SNEXT)+csvLineBreak );
  file.print( "KEY_SPREV="+String(KEY_SPREV)+csvLineBreak );
  file.print( "KEY_LOG="+String(KEY_LOG)+csvLineBreak );
  file.print( "KEY_EDIT0="+String(KEY_EDIT0)+csvLineBreak );
  file.print( "KEY_EDIT1="+String(KEY_EDIT1)+csvLineBreak );
  file.print( "KEY_EDIT2="+String(KEY_EDIT2)+csvLineBreak );
  file.print( "KEY_EDIT3="+String(KEY_EDIT3)+csvLineBreak );
  file.print( "KEY_EDIT4="+String(KEY_EDIT4)+csvLineBreak );
  file.print( "KEY_EDIT5="+String(KEY_EDIT5)+csvLineBreak );
  file.print( "KEY_LNEXT="+String(KEY_LNEXT)+csvLineBreak );
  file.print( "KEY_LPREV="+String(KEY_LPREV)+csvLineBreak );
  /**/
  file.close();
  
  Serial.println("Config::save(): end");

}

void Config::dump(){
  
  File file = SD.open(fileName);                                // öffne CSV-Datei
  if (!file) {                                                  // wenn keine entsprechende CSV-Datei vorhanden ist:
    Serial.println("Config::dump(): file couldn't be opened!"); // Ausgabe der Fehlermeldung
    return;                                                     // aus der Schleife aussteigen
  }
  
  while (file.available())
    Serial.println( file.readStringUntil(csvLineBreak) );

  file.close();

}
