#include <Arduino.h>
#include "Display.h"
#include "Config.h"

Display::Display(){
  
  qtyPressedKeys = 0;
  scrollPosition = 0;
}

void Display::showKeyboard(int index){

  editIndex = index;

  String value = "";

  // getConfigSize()
  // scrollPosition
  // getConfigValues()
  String *values = Config::getConfigValues();
  for( int i = 0; i < Config::getConfigSize();i++)
  {
    if(i == index+scrollPosition){
      value = values[i];
    }
  }
  
  Serial.println( "editing ...:"+value);

  editValue = value;
  showPage(Config::NAME_PAGE_KEYBOARD);
  writeEditValue();
}

void Display::main(){
  if( _isTransceiverConnectedLast != _isTransceiverConnected )
    writeState();

  resetKeys();
  readDisplay();
  
 
  // CONFIGSCREEN
   if(currentPage == Config::NAME_PAGE_SETUP )
  {
    if( pressed(Config::KEY_EDIT0) ){
      showKeyboard(0);
    }   
    else if( pressed(Config::KEY_EDIT1) )
    {
      showKeyboard(1);
    }
    else if( pressed(Config::KEY_EDIT2) )
    {
      showKeyboard(2);
    }
    else if( pressed(Config::KEY_EDIT3) )
    {
      showKeyboard(3);
    }
    else if( pressed(Config::KEY_EDIT4) )
    {
      showKeyboard(4);
    }
    else if( pressed(Config::KEY_EDIT5) )
    {
      showKeyboard(5);
    }
  }

  // KEYBOARDSCREEN
  else if(currentPage == Config::NAME_PAGE_KEYBOARD )
  {
    keyboardLoop();
  }

  _isTransceiverConnectedLast = _isTransceiverConnected;
}

void Display::startWaitPage(){
  returnPage = currentPage;
  showPage(Config::NAME_PAGE_WAIT);
}

void Display::endWaitPage(){
  showPage(returnPage);
}

void Display::resetKeys(){
  qtyPressedKeys = 0;
}

bool Display::pressed(unsigned char key){
  for(int i = 0; i < 16 && i < qtyPressedKeys;i++)
    if (pressedKeys[i] == key )
      return true;

  return false;
}

void Display::readDisplay(){

  unsigned char cmd[16];
  for( int i = 0; i < 16;i++)
    cmd[i] = 0;

  // read whole command from serial bus
  int qtyReceived = 0;
  unsigned long currByte;
  // Code smell: If the transceiver breaks during a command
  //             this ends in an endless loop!!

  while( true ){
    delay(5);
    currByte = Serial3.read();
    if( currByte != -1)
    {
      cmd[qtyReceived++] = currByte;

      if( qtyReceived < 0 || qtyReceived >= 16 )
        return;
      
      // 0x04 0xFF 0xFF 0xFF
      // 0x1A 0xFF 0xFF 0xFF
      if( (cmd[0] == 0x04 || cmd[0] == 0x1A) && cmd[1] == 0xFF && cmd[2] == 0xFF && cmd[3] == 0xFF ){
        for(int i = 4; i < qtyReceived; i++)
          cmd[i-4] = cmd[i];
        qtyReceived -=4;
      }


    }else
      break;
  }
  
  if(qtyReceived == 0)
    return;

  // don't process error messages.
  if( qtyReceived == 0 || cmd[0] == 26 || cmd[0] == 255){
    return;
  }

  if(  cmd[0] == Config::KEY_AUTO
    || cmd[0] == Config::KEY_MAN1
    || cmd[0] == Config::KEY_MAN2
    || cmd[0] == Config::KEY_FREQUENCY_DOWN
    || cmd[0] == Config::KEY_FREQUENCY_UP
    || cmd[0] == Config::KEY_FREQUENCY_STEP
    || cmd[0] == Config::KEY_CAPACITANCE_UP
    || cmd[0] == Config::KEY_CAPACITANCE_DOWN
    || cmd[0] == Config::KEY_INDUCTANCE_UP
    || cmd[0] == Config::KEY_INDUCTANCE_DOWN
    || cmd[0] == Config::KEY_STEP
    || cmd[0] == Config::KEY_SWITCH_CAP_TRX
    || cmd[0] == Config::KEY_SWITCH_CAP_ANT
    || cmd[0] == Config::KEY_STORE
    || cmd[0] == Config::KEY_SETUP
    || cmd[0] == Config::KEY_SSTORE
    || cmd[0] == Config::KEY_EXIT
    || cmd[0] == Config::KEY_SNEXT
    || cmd[0] == Config::KEY_SPREV
    || cmd[0] == Config::KEY_RESET
    || cmd[0] == Config::KEY_EDIT0
    || cmd[0] == Config::KEY_EDIT1
    || cmd[0] == Config::KEY_EDIT2
    || cmd[0] == Config::KEY_EDIT3
    || cmd[0] == Config::KEY_EDIT4
    || cmd[0] == Config::KEY_EDIT5
    || cmd[0] == Config::KEY_SEXIT
    || cmd[0] == Config::KEY_A
    || cmd[0] == Config::KEY_B
    || cmd[0] == Config::KEY_C
    || cmd[0] == Config::KEY_D
    || cmd[0] == Config::KEY_E
    || cmd[0] == Config::KEY_F
    || cmd[0] == Config::KEY_0
    || cmd[0] == Config::KEY_1
    || cmd[0] == Config::KEY_2
    || cmd[0] == Config::KEY_3
    || cmd[0] == Config::KEY_4
    || cmd[0] == Config::KEY_5
    || cmd[0] == Config::KEY_6
    || cmd[0] == Config::KEY_7
    || cmd[0] == Config::KEY_8
    || cmd[0] == Config::KEY_9
    || cmd[0] == Config::KEY_SPACE
    || cmd[0] == Config::KEY_BACK
    
  )
  {
      pressedKeys[qtyPressedKeys++] = cmd[0];
  }else{
      Serial.println("Unknown key: "+String(cmd[0]));
  }
}

// Sends a command to the display
void Display::send(String cmd){
  Serial3.print(cmd);
  Serial3.write(0xFF);
  Serial3.write(0xFF);
  Serial3.write(0xFF);
}

void Display::writeState(){
  if(_isTransceiverConnected)
    send("state.txt=\"polling\"");
  else
    send("state.txt=\"listening\"");
}

void Display::writeStep(unsigned long value) {
  send("sw.txt=\""+String(value)+"\"");
  send("cstep.txt=\""+String(Config::STEP_C_MIN_VALUE*value)+"\"");
  send("lstep.txt=\""+String(Config::STEP_L_MIN_VALUE*value)+"\"");
  step = value;
}

void Display::writeStepFrequency(unsigned long value) {
  send("fsw.txt=\""+String(value)+"\"");
  stepFrequency = value;
}

void Display::writeFrequency(unsigned long value) {
  // We want the frequency to be shown in KHz
  send("freq.txt=\""+String((value/1000.0))+"\"");
  frequency = value;
}

void Display::dim(unsigned char value){
  if( value >100 )
    value = 100;
  send("dim="+String(value) );
}

void Display::writeFrequencyRounded(unsigned long value) {
  send("rfreq.txt=\""+String(value/1000.0)+"\"");
  frequencyRounded = value;
}

void Display::writeC(int value) {
  send("c.txt=\""+String(value*Config::STEP_C_MIN_VALUE)+"\"");
  c = value;
}

void Display::writeL(int value) {
  send("l.txt=\""+String(value*Config::STEP_L_MIN_VALUE)+"\"");
  l = value;
}

void Display::writeDate(String value) {
  send("date.txt=\""+String(value)+"\"");
}

void Display::writeVersion(String value) {
  send("version.txt=\""+String(value)+"\"");
}

void Display::writeLog(String value) {
  send("log.txt=\""+String(value)+"\"");
}

void Display::showPage(String name){
  
  send(name);
  currentPage = name;

  writeStep(step);
  writeStepFrequency(stepFrequency);
  writeFrequency(frequency);
  writeFrequencyRounded(frequencyRounded);
  writeC(c);
  writeL(l);
  writeDate(Config::DATE);
  writeVersion(Config::VERSION);
  writeState();

}

void Display::isTransceiverConected(bool value){
  if( value == _isTransceiverConnected)
    return;

  _isTransceiverConnected = value;
}

void Display::writeEditValue(){
  send("data.txt=\""+editValue+"\"");
}

void Display::keyboardLoop(){
 
  if( pressed(Config::KEY_0) ){
    editValue = editValue + "0";
    writeEditValue();
  }
  else if( pressed(Config::KEY_1) )
  {
    editValue = editValue + "1";
    writeEditValue();
  }
  else if( pressed(Config::KEY_2) )
  {
    editValue = editValue + "2";
    writeEditValue();
  }
  else if( pressed(Config::KEY_3) )
  {
    editValue = editValue + "3";
    writeEditValue();
  }
  else if( pressed(Config::KEY_4) )
  {
    editValue = editValue + "4";
    writeEditValue();
  }
  else if( pressed(Config::KEY_5) )
  {
    editValue = editValue + "5";
    writeEditValue();
  }
  else if( pressed(Config::KEY_6) )
  {
    editValue = editValue + "6";
    writeEditValue();
  }
  else if( pressed(Config::KEY_7) )
  {
    editValue = editValue + "7";
    writeEditValue();
  }
  else if( pressed(Config::KEY_8) )
  {
    editValue = editValue + "8";
    writeEditValue();
  }
  else if( pressed(Config::KEY_9) )
  {
    editValue = editValue + "9";
    writeEditValue();
  }
  else if( pressed(Config::KEY_A) )
  {
    editValue = editValue + "A";
    writeEditValue();
  }
  else if( pressed(Config::KEY_B) )
  {
    editValue = editValue + "B";
    writeEditValue();
  }
  else if( pressed(Config::KEY_C) )
  {
    editValue = editValue + "C";
    writeEditValue();
  }
  else if( pressed(Config::KEY_D) )
  {
    editValue = editValue + "D";
    writeEditValue();
  }
  else if( pressed(Config::KEY_E) )
  {
    editValue = editValue + "E";
    writeEditValue();
  }
  else if( pressed(Config::KEY_F) )
  {
    editValue = editValue + "F";
    writeEditValue();
  }
  else if( pressed(Config::KEY_SPACE) )
  {
    editValue = editValue + " ";
    writeEditValue();
  }
  else if( pressed(Config::KEY_BACK) )
  {
    editValue = editValue.substring(0,editValue.length()-1);
    writeEditValue();
  }
  else if( pressed(Config::KEY_SSTORE) )
  {
    Serial.println("Key pressed: SSTORE");
    String label = "";

    String *labels = Config::getConfigLabels();
    for( int i = 0; i < Config::getConfigSize();i++)
    {
      if(i == editIndex+scrollPosition){
        label = labels[i];
      }
    }    
    
    Config::setConfigValue(label, editValue);
    Config::save();

    showConfig(false);

  }  
  else if( pressed(Config::KEY_SEXIT) )
  {
    showPage(Config::NAME_PAGE_SETUP);
    redrawConfig();
  }  
}

void Display::showConfig( bool reset = true){
  showPage( Config::NAME_PAGE_SETUP);
  if(reset)
    scrollPosition = 0;
    delay(100);
  redrawConfig();
}

void Display::configNext(){
  scrollPosition = min( Config::getConfigSize()-Config::LIST_SIZE, max(0,scrollPosition+Config::SCROLL_WIDTH) );
  redrawConfig();
}

void Display::configPrev(){
  scrollPosition = min( Config::getConfigSize()-Config::LIST_SIZE, max(0,scrollPosition-Config::SCROLL_WIDTH) );
  redrawConfig();
}

void Display::redrawConfig(){

  String *labels = Config::getConfigLabels();
  String *values = Config::getConfigValues();

  for( int i = 0; i < Config::LIST_SIZE; i++){
      send( "label"+ String(i) +".txt=\""+ labels[scrollPosition+i] +"\"");
      send( "value"+ String(i) +".txt=\""+ values[scrollPosition+i] +"\"");
  }

  unsigned char page       = ceil( scrollPosition/(Config::LIST_SIZE*1.0)       )+1;
  unsigned char pageTotals = ceil( Config::getConfigSize()/(Config::LIST_SIZE*1.0) );

  send( "spage.txt=\"Page: "+ String(page) +"/"+ String(pageTotals) +"\"");

}
