#include "Config.h"
#include "LookupTable.h"
#include "Koppler.h"

Koppler::Koppler(){
  valide = true;
  c = NULL;
  l = NULL;
  io = NULL;
  initialized = false;
  invalidate();

}

byte Koppler::getIC1(){
  return _lastA;
}

byte Koppler::getIC2(){
  return _lastB;
}

byte Koppler::getIC3(){
  return _lastC;
}

void Koppler::Koppler::main(){

  if(valide)
    return;

  unsigned int _c0 = c;
  if(Config::INVERT_KOPPLER_OUTPUT_C)
    _c0 = ~_c0;

  unsigned int _io0 = io;
  if(Config::INVERT_KOPPLER_OUTPUT_IO)
    _io0 = ~_io0;

  unsigned int _l0 = l;
  if(Config::INVERT_KOPPLER_OUTPUT_L)
    _l0 = ~_l0;

  // distribution of the output ports IC1
  // lower bits C
  _a = _c0 & B11111111;

  // distribution of the output ports IC2
  //      high bits L         io            high bits C
  _b = _l0>>3&B11100000|_io0<<3&B00011000|_c0>>8&B00000111;

  // distribution of the output ports IC3
  // lower bits L
  _c = _l0 & B11111111;

  if( _lastA != _a || initialized == false ){
    writeBus(Config::IC1,_a);
    delay (300);
  }

  if(_lastB != _b || initialized == false ){
    writeBus(Config::IC2,_b);
      delay (300);
  }

  if(_lastC != _c || initialized == false){
    writeBus(Config::IC3,_c);
    delay (300);
  }

  _lastA = _a;
  _lastB = _b;
  _lastC = _c;
  valide = true;
  initialized = true;
}

void Koppler::setC(int value){
  c = value;
  invalidate();
}

int Koppler::getC(){
  return c;
}

void Koppler::setL(int value){
  l = value;
  invalidate();
}

int Koppler::getL(){
  return l;
}

void Koppler::setIO(unsigned char value){
  io = value;
  invalidate();
}

unsigned char Koppler::getIO(){
  return io;
}

void Koppler::invalidate(){
  valide = false;
}

void Koppler::writeBus(unsigned char addr[8], unsigned char value) {
  
  net.reset();                  // Reset all 1-wire devices
  net.write(0x55);              // Select slave to receive the address

  for (int i = 0; i < 8; i++)   // 8x8  chars read from Array
  {
    net.write(addr[i]);         // send address to slave
  }
  net.write(0x5A);              // Command for sending the address unsigned chars (only the addressed device)
  delay(10);                

  net.write(value);             // write value to slave
  net.write(~value);            // write inverted value to slave

  net.write(0x0089);            // Output values on the 8Ch switch
}
