#include "LookupTable.h"

LookupTable::LookupTable(String fileName) {
  this->fileName = fileName;
  this->csvLineBreak = char(13);
  this->csvSeperator = ";";
}

void LookupTable::dump(String freq){
  
  file = SD.open(fileName);                                 // open CSV-file
  if (!file) {                                              // if no corresponding CSV file is available:
    Serial.println("LookupTable::dump(): File couldn't be opened!.");  // output errormessage
    return;                                                 // get out of the loop
  }
  String line;
  while (file.available()) {
  
    line = file.readStringUntil(csvLineBreak);
    line.trim();
    int comma1 = line.indexOf(csvSeperator);
    String f = line.substring(0, comma1);
    if( freq == "" || f == freq )
    Serial.println( line );
    
  }

  Serial.println( "LookupTable::dump() end!" );
  file.close();

}

bool LookupTable::save(unsigned long frequency, int c, int l, unsigned char io){
  
  String fileNameTemp = "freq.tmp";

  // delete temp file
  if( SD.exists(fileNameTemp) ){
    SD.remove(fileNameTemp);
    Serial.println("LookupTable::save(): Temporary file couldn't be deleted!");
  }
  
  //open original file
  file = SD.open(fileName,FILE_READ);
  if (!file) {
    Serial.println("LookupTable::save(): Config file couldn't be opened!");
    return false;
  }

  // open temp file
  File fileTmp = SD.open(fileNameTemp,FILE_WRITE);
  if (!fileTmp) {
    Serial.println("LookupTable::save() Temporary file couldn't be opened!");
    return false;
  }
  String line;

  bool lineFound = false;
  while (file.available()) {

    line = file.readStringUntil(csvLineBreak);
    line.trim();
    int comma1 = line.indexOf(csvSeperator);
    unsigned long f = line.substring(0, comma1).toInt();

    if(f == frequency){
      lineFound = true;
      //replace line
      line = String(frequency)+csvSeperator+String(c)+csvSeperator+String(l)+csvSeperator+String(io);
    }

    //write original line
    fileTmp.print(line+csvLineBreak);

  }

  // We didn't find find our frequency, so we extend the file.
  // Frequency table hasn't to be sorted by frequency!
  if(lineFound == false)
  {
    line = String(frequency)+csvSeperator+String(c)+csvSeperator+String(l)+csvSeperator+String(io);
    fileTmp.print(line+csvLineBreak);
  }
  
  fileTmp.close();
  file.close();

  // delete original file
  SD.remove(fileName);

  // rename tmp file to original file
  renameFile(fileNameTemp,fileName);
  
  return true;
}

bool LookupTable::renameFile( String oldName, String NewName ){

  // delete temp file
  if( SD.exists(NewName) ){
    SD.remove(NewName);
  }

  //open original file
  file = SD.open(oldName,FILE_READ);
  if (!file) {
    Serial.println("Old File couldn't be opened!");
    return false;
  }

  // open temp file
  File fileTmp = SD.open(NewName,FILE_WRITE);
  if (!fileTmp) {
    Serial.println("New file couldn't be created!");
    return false;
  }

  while (file.available()) {
    String line = file.readStringUntil(csvLineBreak);
    line.trim();
    fileTmp.print(line+csvLineBreak);
  }

  file.close();
  fileTmp.close();

  SD.remove(oldName);
  
  return true;
}

bool LookupTable::hasIOChanged(){
  return io != oldIO;
}

void LookupTable::setFrequency(unsigned long value){

  file = SD.open(fileName);                                 // open CSV-file
  if (!file) {                                              // if no corresponding CSV file is available:
    Serial.println("LookupTable::setFrequency(): File couldn't be opened!");  // output errormessage
    return;                                                 // get out of the loop
  }

  bool foundLine = false;
  while (file.available()) {                                // read CSV-file
    String line = file.readStringUntil(csvLineBreak);       // Read line up to the end of the string

    int comma1 = line.indexOf(csvSeperator);                // split the string up to the first separator ;
    int comma2 = line.indexOf(csvSeperator, comma1 + 1);    // split the string up to the second separator ;
    int comma3 = line.indexOf(csvSeperator, comma2 + 1);    // split the string up to the third separator ;

    unsigned long value1 = line.substring(0, comma1).toInt(); // Convert the 1st substring to int and write to variable value1
    int value2 = line.substring(comma1 + 1, comma2).toInt();  // Convert the 2nd substring to int and write to variable value2
    int value3 = line.substring(comma2 + 1, comma3).toInt();  // Convert the 3rd substring to int and write to variable value3
    unsigned char value4 = line.substring(comma3 + 1).toInt();// Convert the 4th substring to int and write to variable value4
    
    if(value == value1){

      foundLine = true;
      // write the value in the 2nd column (inductance binary value) to C
      C = value2;
      //write the value in the 3rd column (capacity binary value) to L
      L = value3;
      // write the value of the 4th column (position of the capacity) io
      oldIO = io;
      io = value4;

      break;
    }
  }
  file.close();  // close CSV-file
  
  if(foundLine == false){
      C = 0;
      L = 0;
      oldIO = io;
      io = 1;

  }
}

int LookupTable::getC(){
  return this->C;
}

int LookupTable::getL(){
  return this->L;
}

unsigned char LookupTable::getIO(){
  return this->io;
}