#ifndef LookupTable_h
#define LookupTable_h

#include <SD.h>

class LookupTable {

  public:

    // Creates an Instance of the LookupTale object.
    // @param fileName        The file name of the CSV-File on the SD-Card.
    // @param pin             The pin the card module is connected to.
    LookupTable(String fileName);

    // Sets the frequency to get the values of.
    // @param value           The frequency to look for.
    void setFrequency(unsigned long value);

    // Returns the C value of the frequency set by setFrequency()
    int getC();

    // Returns the L value of the frequency set by setFrequency()
    int getL();

    // Returns the IO value of the frequency set by setFrequency()
    unsigned char getIO();


    // Returns if the IO has changed after a setFrequency call.
    bool hasIOChanged();

    // Saves the values of l, c, io to the given frequency.
    // @param frequency           The frequency to store the values to.
    // @param l                   The l value to store.
    // @param c                   The c value to store.
    // @param io                  The io value to store.
    bool save(unsigned long frequency, int l, int c, unsigned char io);

    // Renames a file on the SD card.
    // This function is very stinky.
    // Actually it copies the file and stores it under the new name
    // and removes the old name.
    // TODO: Clean Code smell
    bool renameFile( String oldName, String NewName );

    // Dumps the csv file on the Serial port.
    // @param freq       The frequency to dump.
    //                   freq is optional. If '' is given
    //                   the whole file is dumped.
    void dump(String freq);

  private:
    // The line break character in the csv file.
    char csvLineBreak;
    
    // The column seperator in the csv file.
    String csvSeperator;

    // The file name if the csv file on the csv file.
    String fileName;

    // A file handle
    File file;

    // The actual read C value.
    int C;

    // The actual read L value.
    int L;

    // The actual read io value.
    unsigned char io;

    // The last io value read of the file.
    unsigned char oldIO;
};

#endif